%Tutorial 1: Orthonormal Bases

%This is a tutorial on a specific set of linear transformations called
%basis projections. The conditions for an [N N] matrix to be an orthonormal
%basis, and what that implies with respect to data analysis.


% %% BEGIN SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % 
% %   SECTION 1: Creating Random 2D Data and example 2D Bases.
%     clear;
%     
%     %N      :   The number of data points.
%     %c      :   The correlation between the two dimensions.
%     %D      :   The data matrix, [observations samples].
% 
%     N = 100;
%     c = .3;
% 
%     %Making random data
%     r1 = [randn(1,floor(N/2))-2 randn(1,ceil(N/2))+2];
%     randn('state',sum(100*clock));
%     r2 = [randn(1,floor(N/2))-2 randn(1,ceil(N/2))+2];
%     r = [r1; r2];
% 
%     %Making correlated data
%     C = [1 c/2; c/2 1]; 
%     D = C*r;
% 
%     %Computing the Basis using SVD
%     %PC :   The Principle Components (not used).
%     %E  :   The Energy Matrix (not used).
%     %B  :   The SVD Basis.
%     %DP :   The data D projected onto the basis.
% 
%     [PC E B] = svd(D');
%     sB = [sign(B(1,1)) sign(B(2,2))];
%     B = B.*[sB; sB];
%     DP = B'*D;
% 
%     %Computing the Rotation Matrix Basis
%     %theta  :   The rotation angle.
% 
%     theta = pi/(2.^5);
%     R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
% 
% 
%     %Displaying the Basis Property: B'*B = I.
%     sp = [' '; ' '];
%     sp = [sp sp sp sp];
%     disp(' ');
%     disp('Multiplying a Basis and its transpose yields the identity matrix.');
%     disp(' ');
%     disp('The SVD basis B for the data: B''*B');
%     disp(' ');
%     disp([sp num2str(B'*B,3)]);
%     disp(' '); disp(' ');
%     disp('The Rotation Basis R in two dimensions: R''*R');
%     disp(' ');
%     disp([sp num2str(R'*R,3)]);
%     disp(' '); disp(' ');
% 
%     %Plotting the data and the basis and rotating both by theta.
% 
%     %Instructions:
%     disp('Press -> to advance the rotation.');
%     disp('Press: CTRC + C to exit the loop.');
% 
%     %Initializing the figure, initial plotting, and computing the viewing window.
%     figure;
%     setP([393 526 1523 476]);
%     A = [max(abs(minmax(flatten(D))))];
%     A = [-A A -A A];
%     subplot(1,3,1);
%         scatter(D(1,:),D(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 B(1,1)],[0 B(2,1)],'k',[0 B(1,2)],[0 B(2,2)],'k','LineWidth',2); hold off;
%         title('Original Data B with Basis Vectors B');
%         axis(A); axis square;
%     subplot(1,3,3);
%         scatter(DP(1,:),DP(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 1],[0 0],'k',[0 0],[0 1],'k','LineWidth',2); hold off;
%         title('Original Data with Basis Vectors');
%         title('Projecting D onto basis B: B''*D');
%         axis(A); axis square;
%     subplot(1,3,2);
%         scatter(D(1,:),D(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 B(1,1)],[0 B(2,1)],'k',[0 B(1,2)],[0 B(2,2)],'k','LineWidth',2); hold off;
%         title(['Rotating D and B by ' 0 '*\theta:  R''D']);
%         axis(A); axis square;
% 
%     %The rotation loop.
%     k = 0;
%     BR = B;
%     DR = D;
%     done = 0;
%     while(~done);
%         subplot(1,3,2);
%         scatter(DR(1,:),DR(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 BR(1,1)],[0 BR(2,1)],'k',[0 BR(1,2)],[0 BR(2,2)],'k','LineWidth',2); hold off;
%         title(['Rotating D and B by ' int2str(k) '*\theta:  R''D']);
%         axis(A); axis square;
%         BR = R'*BR;
%         DR = R'*DR;
%         pause;
%         k = k + 1;
%         figure(gcf);
%     end
%     
%     % QUESTIONS:
%     %
%     % 1) Explain how the SVD basis B relates to rotation? 
%     %
%     % 2) Do you think the relationship in 1) generalizes to all basis 
%     %    projections P, where P satisfies P'*P = I? Explain your reasoning.
%     %
%     % 3) Geometrically speaking, how do the SVD basis vectors transform the 
%     %    data D? What mathematical property of D might this correspond to?
% 
%     
% %% END SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%% COMMENT OUT SECTION 1. UNCOMMENT SECTION 2 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


%% BEGIN SECTION 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   SECTION 2: Multiple Bases and Rotating Shapes.
    clear;

    %S      :   The indices of a shape; the tetrahedron.
    %R1     :   The rotation matrix for dimension 1.
    %R2     :   The rotation matrix for dimension 2.
    %R3     :   The rotation matrix for dimension 3.
    %theta  :   The rotation angles: [theta(1) theta(2) theta(3)].
    %p      :   A proportion in [0 1].
    
    %The rotation angles
    p = .1;
    theta = p*[-pi/(2.^4) pi/(2.^1) pi/(2.^8)];
        
    %The tetrahedron
    S = [1 1 1; 1 -1 -1; -1 1 -1; -1 -1 1];
    ind = [1 2 3 1 3 4 1 4 2];
    S = S(ind,:)';
    
    %The rotation matrices
    R1 = [1 0 0; 0 cos(theta(1)) -sin(theta(1)); 0 sin(theta(1)) cos(theta(1))];
    R2 = [cos(theta(2)) 0 sin(theta(2)); 0 1 0; -sin(theta(2)) 0 cos(theta(2))];
    R3 = [cos(theta(3)) -sin(theta(3)) 0; sin(theta(3)) cos(theta(3)) 0; 0 0 1];
    
    %Displaying the Basis Property: B'*B = I.
    sp = [' '; ' '; ' '];
    sp = [sp sp sp sp];
    disp(' ');
    disp('Multiplying a Basis and its transpose yields the identity matrix.');
    disp(' ');
    disp('Rotation Matrix R1''*R1 = I:');
    disp(' ');
    disp([sp num2str(R1'*R1,1)]);
    disp(' '); disp(' ');
    disp('Rotation Matrix R2''*R2 = I:');
    disp(' ');
    disp([sp num2str(R2'*R2,1)]);
    disp(' '); disp(' ');
    disp('Rotation Matrix R3''*R3 = I:');
    disp(' ');
    disp([sp num2str(R3'*R3,1)]);
    disp(' '); disp(' ');
    
    %Making a movie of the rotating the tetrahedron.
    %
    %mov        :   A data structure for making avi files.
    %filename   :   The name of the movie file.
    %framerate  :   The movie framerate in samples/sec.
    %length     :   Length of the movie in seconds.
    
    length = 2;
    framerate = 100;
    filename = 'Tetrahedron';
    N = length*framerate;
    
    %Plotting the initial tetrahedron.
    figure;
    setP([392 412 706 594]);
        h = plot3(S(1,:),S(2,:),S(3,:),'Color',[0 0 .8],'LineWidth',2);
        title('Continuous rotation of the tetrahedron by \theta');
        axis square; axis([-2 2 -2 2 -2 2]); grid on;
    
    %Initializing the movie.
    set(gcf,'DoubleBuffer','on');
    mov = avifile(filename,'fps',framerate);
    mov.Quality = 25;
    
    %The movie loop
    SR = S;
    for k = 1:N;
        %Updating the axis.
        set(h,'XData',SR(1,:));
        set(h,'YData',SR(2,:));
        set(h,'ZData',SR(3,:));
        
        %Rotating the tetrahedron.
        SR = R1'*SR;
        SR = R2'*SR;
        SR = R3'*SR;
        
        %Collecting a frame for the movie.
        frame{k} = getframe(gca);
    end
    close(gcf); 
    pause(.01);
    
    %Compiling the frames.
    disp('Compiling Movie...');
    str = 'mov = addframe(mov';
    for k = 1:N
        str = [str ',frame{' int2str(k) '}']; 
    end
    str = [str ');'];
    warning off;
    eval(str);
    warning on;
    
    %Closing the movie.
    mov = close(mov);
    avireset;
    
    % QUESTIONS:
    %
    %   Suppose you were given 2 N-dimensional vectors:
    %    
    %        X = [x1 ... xN];
    %        Y = [y1 ... yN];
    %
    %   The angle theta between X and Y in [0 pi] is 
    %
    %        theta = acos((X*Y')./((X*X')*(Y*Y')));
    %   
    %   1) What condition needs to be true for there to exist a rotation
    %      that can map X onto Y?
    %
    %   2) In matlab notation and based on the above syntax, define the 
    %      data matrix D in terms of X and Y, and list the properties of 
    %      R such that R'*D is a valid rotation of D.
    %      
    %      In terms of D, what must be true about R'*D so that R rotates 
    %      X to Y?
    %
    %  *3) How might you find rotation matrices 
    %
    %        R(1,theta(1)) , ... , R(N-1,theta(N-1))
    %
    %      along with the corresponding angles 
    %       
    %        [theta(1) ... theta(N-1)]
    % 
    %      such that rX'*X = Y, where rX is the successive rotation of 
    %      X, defined in the following way:
    %           
    %        rX = I;
    %        for k = 1:N-1
    %           rX = R(k,theta(k))'*rX;
    %        end
    %
    %     where
    %
    %     rX'*X = Y.
    % 
    %     This question is HARD! I don't expect anyone to arrive at a
    %     complete solution, but it is a problem that is worth
    %     considering in the context of high-dimensional data analysis. 
    %
    %     Think about rotating around a single plane each time, but using
    %     an arbitrary [N-2 N-2] orthonormal basis to define the rest of
    %     the matrix. Choose the easiest basis to yield a solution.


%% END SECTION 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%