%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Tutorial: Linear Systems %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% This tutorial provides background information concerning linear systems.
%% Topics covered are: 
%%
%%    1) Basis vectors and basis functions.
%%    2) The Fourier basis.
%%    3) Singular-Value Decomposition.
%%    4) The convolution operator.
%%    5) Time-invariant linear systems and the impulse response function.
%%
%% The tutorial requires commenting and uncommenting of different blocks of
%% code, at the specified markers. This can be done by: 
%%
%%    -High-lighting the block of text to uncomment.
%%
%%    -Going to the TEXT tab at the top of the window, and pressing
%%     UNCOMMENT. (In windows, the shortcut for this operation is CTRL+T).
%%   
%%    -To comment the code, go to the TEXT tab and press COMMENT. (In windows the
%%     shortcut for this operation is CTRL+R).
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% BEGIN SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   
    %   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   %% SECTION 1: Orthonormal Bases %%
    %   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    %   This is a tutorial on a specific set of linear transformations 
    %   called basis projections. The conditions for an [N N] matrix to be 
    %   an orthonormal basis, and what that implies with respect to data 
    %   analysis.
    %
    %   Any orthonormal basis B has the following properties:
    %
    %   1) B*B' = I
    %   2) B' = inv(B)
    %
    %   If B(k,:) is taken to be the basis vector k. and 
    %   size(B,2) = size(D,1), projecting the data onto B is written:
    %
    %   R*D
    %
    %   This operation serves to rotate the coordinates of D. Getting an
    %   intuition for this will be the subject of this first section.
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 1.1: Creating Random 2D Data and example 2D Bases. %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   
%     %N      :   The number of data points.
%     %c      :   The correlation between the two dimensions.
%     %D      :   The data matrix, [observations samples].E
%     
%     %Initializing Variables
%     clear;
%     N = 100;
%     c = .3;
% 
%     %Making random data
%     r1 = [randn(1,floor(N/2))-2 randn(1,ceil(N/2))+2];
%     randn('state',sum(100*clock));
%     r2 = [randn(1,floor(N/2))-2 randn(1,ceil(N/2))+2];
%     r = [r1; r2];
% 
%     %Making correlated data
%     C = [1 c/2; c/2 1]; 
%     D = C*r;
% 
%     %Computing the Basis using SVD
%     %PC :   The Principle Components (not used).
%     %E  :   The Energy Matrix (not used).
%     %B  :   The SVD Basis.
%     %DP :   The data D projected onto the basis.
%     
%     %Introducing correlations
%     [PC E B] = svd(D');
%     B = B';
%     sB = [sign(B(1,1)) sign(B(2,2))];
%     B = B'.*[sB; sB];
%     DP = B*D;
% 
%     %Computing the Rotation Matrix Basis
%     %theta : the rotation angle.
%     theta = pi/(2.^5);
%     R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
% 
%     %Displaying the Basis Property: B*B' = I.
%     sp = [' '; ' '];
%     sp = [sp sp sp sp];
%     disp(' ');
%     disp('Multiplying a Basis and its transpose yields the identity matrix.');
%     disp(' ');
%     disp('The SVD basis B for the data: B''*B');
%     disp(' ');
%     disp([sp num2str(B'*B,3)]);
%     disp(' '); disp(' ');
%     disp('The Rotation Basis R in two dimensions: R''*R');
%     disp(' ');
%     disp([sp num2str(R'*R,3)]);
%     disp(' '); disp(' ');
% 
%     %Plotting the data and the basis and rotating both by theta.
%     %Instructions:
%     disp('Press -> to advance the rotation.');
%     disp('Press: CTRC + C to exit the loop.');
% 
%     %Initializing the figure, initial plotting, and computing the viewing window.
%     figure;
%     setP([393 526 1523 476]);
%     A = [max(abs(minmax(flatten(D))))];
%     A = [-A A -A A];
%     subplot(1,3,1);
%         scatter(D(1,:),D(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 B(1,1)],[0 B(2,1)],'k',[0 B(1,2)],[0 B(2,2)],'k','LineWidth',2); hold off;
%         title('Original Data B with Basis Vectors B');
%         axis(A); axis square;
%         setXT; setXTL;
%         setYT; setYTL;
%     subplot(1,3,3);
%         scatter(DP(1,:),DP(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 1],[0 0],'k',[0 0],[0 1],'k','LineWidth',2); hold off;
%         title('Original Data with Basis Vectors');
%         title('Projecting D onto basis B: B''*D');
%         xlabel('Observe that the new basis is a rotation of the original basis');
%         axis(A); axis square;
%         setXT; setXTL;
%         setYT; setYTL;
%     subplot(1,3,2);
%         scatter(D(1,:),D(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 B(1,1)],[0 B(2,1)],'k',[0 B(1,2)],[0 B(2,2)],'k','LineWidth',2); hold off;
%         title(['Rotating D and B by ' 0 '*\theta:  R''D']);
%         axis(A); axis square;
%         setXT; setXTL;
%         setYT; setYTL;
% 
%     %The rotation loop.
%     k = 0;
%     BR = B;
%     DR = D;
%     done = 0;
%     while(~done);
%         subplot(1,3,2);
%         scatter(DR(1,:),DR(2,:),3,[0 0 .8],'filled'); hold on;
%         plot([0 BR(1,1)],[0 BR(2,1)],'k',[0 BR(1,2)],[0 BR(2,2)],'k','LineWidth',2); hold off;
%         title(['Rotating D and B by ' int2str(k) '*\theta:  R''D']);
%         axis(A); axis square;
%         setXT; setXTL;
%         setYT; setYTL;
%         BR = R'*BR;
%         DR = R'*DR;
%         pause;
%         k = k + 1;
%         figure(gcf);
%     end
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 1.1: QUESTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %     
    %    QUESTIONS:
    %     
    %    1) Describe the action of B on data D with respect to rotation. 
    %    
    %    2) Do you think the relationship in 1) generalizes to all basis 
    %       projections P, where P is an NxN matrix satisfying P*P' = I? 
    %       Explain your reasoning.
    % 
    %    3) Geometrically speaking, what does the SVD bases seem to do to 
    %       the data D. What mathematical property of D might this 
    %       correspond to?
    %     
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
    %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 1.2: Multiple Bases and Rotating Shapes. %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %S      :   The indices of a shape; the tetrahedron.
    %R1     :   The rotation matrix for dimension 1.
    %R2     :   The rotation matrix for dimension 2.
    %R3     :   The rotation matrix for dimension 3.
    %theta  :   The rotation angles: [theta(1) theta(2) theta(3)].
    %p      :   A proportion in [0 1].
    
    %THE ROTATION ANGLES: play around with them
    clear;
    p = .1;
    theta = p*[-pi/(2.^4) pi/(2.^1) pi/(2.^8)];
        
    %The tetrahedron
    S = [1 1 1; 1 -1 -1; -1 1 -1; -1 -1 1];
    ind = [1 2 3 1 3 4 1 4 2];
    S = S(ind,:)';
    
    %The rotation matrices
    R1 = [1 0 0; 0 cos(theta(1)) -sin(theta(1)); 0 sin(theta(1)) cos(theta(1))];
    R2 = [cos(theta(2)) 0 sin(theta(2)); 0 1 0; -sin(theta(2)) 0 cos(theta(2))];
    R3 = [cos(theta(3)) -sin(theta(3)) 0; sin(theta(3)) cos(theta(3)) 0; 0 0 1];
    
    %Displaying the Basis Property: B'*B = I.
    sp = [' '; ' '; ' '];
    sp = [sp sp sp sp];
    disp(' ');
    disp('Multiplying a Basis and its transpose yields the identity matrix.');
    disp(' ');
    disp('Rotation Matrix R1''*R1 = I:');
    disp(' ');
    disp([sp num2str(R1'*R1,1)]);
    disp(' '); disp(' ');
    disp('Rotation Matrix R2''*R2 = I:');
    disp(' ');
    disp([sp num2str(R2'*R2,1)]);
    disp(' '); disp(' ');
    disp('Rotation Matrix R3''*R3 = I:');
    disp(' ');
    disp([sp num2str(R3'*R3,1)]);
    disp(' '); disp(' ');
    
    %Making a movie of the rotating the tetrahedron.
    %mov        :   A data structure for making avi files.
    %filename   :   The name of the movie file.
    %framerate  :   The movie framerate in samples/sec.
    %length     :   Length of the movie in seconds.
    
    length = 4;
    framerate = 100;
    filename = 'Tetrahedron';
    N = length*framerate;
    delete('tetrahedron.avi');
    
    %Plotting the initial tetrahedron.
    figure;
    setP([392 412 706 594]);
        h = plot3(S(1,:),S(2,:),S(3,:),'Color',[0 0 .8],'LineWidth',2);
        title('Continuous rotation of the tetrahedron by \theta');
        axis square; axis([-2 2 -2 2 -2 2]); grid on;
        setXTL; setYTL; setZTL;
    
    try
    %Initializing the movie.
    set(gcf,'DoubleBuffer','on');
    mov = avifile(filename,'fps',framerate);
    mov.Quality = 25;
    
    %The movie loop
    SR = S;
    for k = 1:N;
        %Updating the axis.
        set(h,'XData',SR(1,:));
        set(h,'YData',SR(2,:));
        set(h,'ZData',SR(3,:));
        
        %Rotating the tetrahedron.
        SR = R1'*SR;
        SR = R2'*SR;
        SR = R3'*SR;
        
        %Collecting a frame for the movie.
        frame{k} = getframe(gca);
    end
    close(gcf); 
    pause(.01);
    
    %Compiling the frames.
    disp('Compiling Movie...');
    str = 'mov = addframe(mov';
    for k = 1:N
        str = [str ',frame{' int2str(k) '}']; 
    end
    str = [str ');'];
    warning off;
    eval(str);
    warning on;
    
    %Closing the movie.
    mov = close(mov);
    avireset;
    catch
        clear mex;
        delete('tetrahedron.avi');
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     % SECTION 1.2: QUESTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     
%       Suppose you were given 2 N-dimensional vectors:
%        
%            X = [x1 ... xN];
%            Y = [y1 ... yN];
%     
%       The angle theta between X and Y in [0 pi] is 
%     
%            theta = acos((X*Y')./((X*X')*(Y*Y')));
%       
%       1) What condition needs to be true for there to exist a rotation
%          that can map X onto Y?
%     
%       2) In matlab notation and based on the above syntax, define the 
%          data matrix D in terms of X and Y, and list the properties of 
%          R such that R'*D is a valid rotation of D.
%          
%          In terms of D, what must be true about R'*D so that R rotates 
%          X to Y?
%     
%      *3) How might you find rotation matrices 
%     
%            R(1,theta(1)) , ... , R(N-1,theta(N-1))
%     
%          along with the corresponding angles 
%           
%            [theta(1) ... theta(N-1)]
%     
%          such that rX'*X = Y, where rX is the successive rotation of 
%          X, defined in the following way:
%               
%            rX = I;
%            for k = 1:N-1
%               rX = R(k,theta(k))'*rX;
%            end
%     
%         where
%     
%         rX'*X = Y.
%     
%         This question is HARD! I don't expect anyone to arrive at a
%         complete solution, but it is a problem that is worth
%         considering in the context of high-dimensional data analysis. 
%     
%         Think about rotating around a single plane each time, but using
%         an arbitrary [N-2 N-2] orthonormal basis to define the rest of
%         the matrix. Choose the easiest basis to yield a solution.
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% BEGIN SECTION 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 2: Decomposing waves using SVD. %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Previously, we defined data D = [observations time], where observations
%   where simultanoeusly sampled data. In space-time analysis, the data 
%   matrix has the follow dimensions:
%
%       D = [space time]
%
%   Where the observations are taken over some regular partition of space.
%   This can be done in any number of dimensions, but due to discrete 
%   sampling, can still be thought of in one dimension.
%
%   The SVD decomposition is a factorization of an arbitrary data D in the
%   following way:
%
%       D = S*E*T';
%
%   It is a unique decomposition with virtually constraints on the matrices 
%   it applies to, much like the existence of a prime decomposition of a
%   natural number (0,1,2,3,4,...). A statement of the existence of such a
%   factorization is known as the spectral theorem. The interested reader
%   can check the wikipedia page on the subject, which is rigorous:
%
%   http://en.wikipedia.org/wiki/Spectral_theorem
%
%   Suppose S(k,:) is the kth spatial eigenvectors, T(k,:) is the kth 
%   temporal component, and E is a diagonal energy (eigenvalue) matrix.
%
%   T also has the property that T*T' = I, the identity matrix. This 
%   property is equivalent to the fact that the temporal components of the 
%   decomposition also form an orthonormal bases.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         %% SECTION 2.1: Reconstrunction and Filtering with SVD %%
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     
%         %Initializing a random matrix with some structure.
%         [X Y] = meshgrid(linspace(-3,3,100),linspace(-5,5,10));
%         r = rand(10,100);
%         D = r + 10*normpdf(X,0,1).*normpdf(Y,2,2);
%         
%         %Performing Matrix Decomposition.
%         [S E T] = svd(D);
%         S = S'; T = T';
%         DD = S'*E*T;
%     
%         %Performing the SVDfilter.
%         Dfilt = SVDfilter(D,1);
%         
%         %Displaying the original matrix and the reconstruction.
%         figure;
%         setP([281 289 996 416]);
%         subplot(1,3,1);
%             imagesc(D); colormap(ice);
%             title('Sample Dataset');
%             setXT; setYT; setXTL; setYTL; setYD;
%         subplot(1,3,2);
%             imagesc(S'*E*T); colormap(ice);
%             title('SVD reconstruction: S''*E*T');
%             setXT; setYT; setXTL; setYTL; setYD;
%         subplot(1,3,3)
%             imagesc(Dfilt); colormap(ice);
%             title('Filtered data D');
%             setXT; setYT; setXTL; setYTL; setYD;
%     
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 2.2: 3D Reconstrunction of waves with SVD %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     
%     %Creating a 3-Dimensional data matrix as the sum of two propagating 
%     %sinusoidal gratings. They will then be decomposed using SVD.    
%     N = 25;
%     t = 1:100; 
%     [X Y Z] = meshgrid(linspace(0,1.5*pi/2,N),linspace(0,1.5*pi/2,N),linspace(0,6*pi,length(t)));
%     data1 = cos(X + Z)+cos(Y + Z);
%     
%     [X Y Z] = meshgrid(linspace(0,6*pi,N),linspace(0,6*pi/2,N),linspace(0,12*pi,length(t)));
%     data2 = cos(X + 3*Z)+cos(Y - 3*Z);
%     D = data1 + .25*data2;
%     
%     %Demonstrating filtering with SVD.
%     s = size(D);
%     datatemp = zeros(s(1)*s(2),s(3));
%     for k = 1:s(1)
%         datatemp(((k-1)*s(1) + 1):k*(s(1)),:) = D(k,:,:);
%     end
%     
%     [dataSVD11 B2 E] = SVDfilter(datatemp,2);
%     [dataSVD22 B2 E] = SVDfilter(datatemp,10);
%     dataSVD22 = dataSVD22-dataSVD11;
%     
%     dataSVD1 = repmat(nan,size(D));
%     dataSVD2 = repmat(nan,size(D));
%     B = zeros(N,N,N*N);
%     for k = 1:s(1)
%         dataSVD1(k,:,:) = dataSVD11(((k-1)*s(1) + 1):k*(s(1)),:);
%         dataSVD2(k,:,:) = dataSVD22(((k-1)*s(1) + 1):k*(s(1)),:);
%     end
%     for n = 1:N*N
%         temp = zeros(N,N);
%         for k = 1:N;
%             temp(:,k) = B2(((k-1)*s(1) + 1):k*(s(1)),n);
%         end
%         B(:,:,n) = temp;
%     end
%     
%     figure;
%     setP([5 384 1654 697]);
%     done = 0;
%     k = 1;
%     n = 1;
%     while(~done)
%         subplot(2,4,1);
%             imagesc(data1(:,:,k));
%             colormap(ice);
%             axis square; setXTL; setYTL;
%             title('Wave1');
%         subplot(2,4,2);
%             imagesc(data2(:,:,k));
%             colormap(ice);
%             axis square; setXTL; setYTL;
%             title('Wave2');
%             setXT; setXTL;
%             setYT; setYTL;
%         subplot(1,2,2);
%             imagesc(data(:,:,k));
%             colormap(ice);
%             axis square; setXTL; setYTL;
%             title('Wave1 + Wave2');
%             setXT; setXTL;
%             setYT; setYTL;
%             xlabel(int2str(n));
%         subplot(2,4,5);
%             imagesc(dataSVD1(:,:,k));
%             colormap(ice);
%             axis square; setXTL; setYTL;
%             title('SVDFilter1 : 1-2');
%             setXT; setXTL;
%             setYT; setYTL;
%         subplot(2,4,6);
%             imagesc(dataSVD2(:,:,k));
%             colormap(ice);
%             axis square; setXTL; setYTL;
%             title('SVDFilter1 : 3-4');
%             setXT; setXTL;
%             setYT; setYTL;
%         pause(.025);
%         k = mod(k,length(t)) + 1;
%         n = n+1;
%         if(n > 250)
%             done = 1;
%         end
%     end
%     
%     subplot(2,2,1); 
%         imagesc(B(:,:,1));
%         colormap(ice);
%         title('Spatial Eigenvector 1');
%         setXT; setXTL;
%         setYT; setYTL;
%     subplot(2,2,2); 
%         imagesc(B(:,:,2));
%         colormap(ice);
%         title('Spatial Eigenvector 2');
%         setXT; setXTL;
%         setYT; setYTL;
%     subplot(2,2,3); 
%         imagesc(B(:,:,3));
%         colormap(ice);
%         title('Spatial Eigenvector 3');
%         setXT; setXTL;
%         setYT; setYTL;
%     subplot(2,2,4); 
%         imagesc(B(:,:,4));
%         colormap(ice);
%         title('Spatial Eigenvector 4');
%         setXT; setXTL;
%         setYT; setYTL;
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 2: Questions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   1) In example 1, the data was a Gaussian curve with additive noise.
    %      With respect to SVD, what is the dimensionality of the Gaussian
    %      curve?
    %
    %      Look at the eigenvectors used generated in SVD. How many are
    %      there? 
    %
    %      Therefore, what dimensionality is the noise?
    %   
    %   2) How many waves are represented in the dataset constructed for
    %      example 2?
    %
    %      How many eigenvectors are needed to reconstruct each?
    %
    %      Look at Spatial Eigenvector 1 and Spatial Eigenvector 2. What
    %      two properties of the first wave is each component representing?
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% BEGIN SECTION 3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 3: The Fourier Transform %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     This is a tutorial on a particularly well-studied set of basis 
%     functions that give rise to the Fourier series and the Fourier 
%     transform. Emphasis is placed on the implicit role of periodicity 
%     in applying the Fourier transform to time-series data. 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 3.1: The Oddities of Periodicity %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %% The period functional Per(f) and how it varies for Per(f+g).
%     %%
%     %% Period is the (non-negative) inverse of frequency. Therefore, if 
%     %% f = sin(wt), the Per(f) = abs(1/w).
%     %%
%     %% Suppose we had a series of periodic function [f1 ... fN] with
%     %% frequencies [1 ... N]. Then 
%     %%
%     %% [Per(f1) ... Per(fN)] = [abs(1/1) ... abs(1/N)].
%     %%
%     %% If f and g are periodic functions, it is almost always true that: 
%     %%   
%     %% Per(f + g) = lcm(Per(f),Per(g)).
%     %%
%     %% where lcm(m,n) is the least common multiple function extended to the 
%     %% rationals.
%     
%     %N      :   The number of periodic function.
%     %w      :   The frequency of each function in [1 N].
%     %t      :   A variable representing time.
%     %F      :   F(w,:) = fw = sin(w*t).
%     
%     %Constructing F(w,t).
%     %Play around with the number of sinusoids N.
%     %The lcm function returns false results when the integers get too large.
%     clear;
%     N = 10;
%     t = linspace(0,2*pi,1000);
%     F = zeros(N,length(t));
%     for w = 1:N
%         F(w,:) = sin(w*t);
%     end
%     
%     %Constructing PerF = c*Per(F(w,:)), where c = factorial(N). Values in
%     %PerF are natural numbers so as to be compatible with lcm. The true 
%     %period of F(w,:) = PerF(w)/c.
%     %
%     %PerFF(i,j) = Per(F(i,:) + F(j,:)) = lcm(Per(F(i,:)),Per(F(j,:))).
%     
%     w = [1:N];
%     c = factorial(N);
%     PerF = zeros(1,N);
%     for k = 1:N
%         PerF(k) = prod(setdiff(w,k));
%     end
%     PerFF = zeros(N,N);
%     for i = 1:N
%         for j = 1:i
%             PerFF(i,j) = lcm(PerF(i),PerF(j))/c;
%             PerFF(j,i) = PerFF(i,j);
%             if(lcm(PerF(i),PerF(j))/c < 0)
%                 tt = 1;
%             end
%         end
%     end
%     
%     %Visualizing F and PerFF.
%     figure;
%     setP([393 502 1526 502]);
%     subplot(1,2,1);
%         imagesc(t,w,F); h = colorbar;
%         title([int2str(N) ' sinusoids with integral-valued frequencies']);
%         setXTL; setYT(w); setYTL(w);
%         xlabel('Time');
%         ylabel('Frequency w');
%         cblabel(h,'Height');
%     subplot(1,2,2);
%         imagesc(w,w,PerFF); h = colorbar;
%         title('The periods of sums of sinusoids: PerFF(w1,w2)');
%         xlabel('Frequency = w1');
%         ylabel('Frequency = w2');
%         cblabel(h,'Period');
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 3.1: Questions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   1) Open a new script called garbage, by typing 'edit garbage' in the
    %      command line. Suppose you had a sequence of periodic functions G
    %      with periods (not frequencies) PerG = [1 ... 50]. Compute PerGG 
    %      and display it using the imagesc function. Comment on the 
    %      aesthetic nature of the lcm function.    
    %
    %   2) Let eps > 0 be a small number that you imagine to take on any 
    %      value close to zero. For an arbitrary w, as eps => 0 is 
    %       
    %      P(w,eps) = Per(sin(wt) + sin((w + eps)t))
    %
    %      well-behaved? Why or why not?
    %
    %      Hint: Consider all natural numbers k, such that eps = w/k.
    %      Construct the behavior of more sequences of w by considering the 
    %      initial frequency to change from w to 1/P(w,eps), for any 
    %      eps = w/k.
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 3.2: The Fourier Basis %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %% The main idea behind the Fourier transform is that sines and cosines
%     %% of specific frequencies are a set of orthonormal basis functions. The
%     %% goal is to examine the complex exponential, and its relationship to 
%     %% sines and cosines via Euler's identity:
%     %%
%     %% a*exp(i*w*t + theta) = a*cos(w*t + theta) + i*a*sin(w*t + theta);
%     
%     % N         :   N is the number of data points in the time series.
%     % t         :   A time variable.
%     % a         :   An amplitude.
%     % w         :   A frequency.
%     % theta     :   An initial phase.
%     % f         :   A complex exponential.
%     % F         :   An [N N] orthonormal basis.
%     
%     %initializing a complex exponential.
%     N = 200;
%     a = 1;
%     w = 1;
%     theta = 0;
%     t = linspace(0,2*pi,N);
%     f = a*exp(2*pi*sqrt(-1)*w*t + theta);
%     
%     %Plotting the complex exponential in 2 and 3 dimensions.
%     h = figure;
%     setP([99 302 1177 418]);
%     subplot(1,2,1);
%         plot(real(f),imag(f),'Color',[0 0 .8],'LineWidth',2);
%         axis([-a-a/10 a+a/10 -a-a/10 a+a/10]);
%         axis square; setXT([-a 0 a]); setYT([-a 0 a]);
%         title('The Complex Exponential');
%         xlabel('real(f)');
%         ylabel('imag(f)');
%     subplot(1,2,2)
%         plot3(t,real(f),imag(f),'Color',[0 0 .8],'LineWidth',2);
%         axis([0 2*pi -a-a/10 a+a/10 -a-a/10 a+a/10]);
%         setXT; setYT([-a 0 a]); setZT([-a 0 a]);
%         title('The Complex Exponential Helix');
%         xlabel('time');
%         ylabel('real(f)');
%         zlabel('imag(f)');
%     disp('Press ANY KEY to move on to the next figure.')
%     pause;
%         
%     %initializing the Fourier basis.
%     N = 50;
%     i = sqrt(-1);
%     w = [0:N-1]./N;
%     t = [0:N-1];
%     F = zeros(N,N);
%     for k = 1:N;
%         F(k,:) = sqrt((1/N))*exp(2*pi*i*w(k)*t);
%     end
%     
%     figure; 
%     setP([394 46 1175 956]);
%     subplot(2,2,1);
%         imagesc(real(F));
%         colorbar;
%         title('Fourier Basis:  real(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,3);
%         imagesc(imag(F));
%         colorbar;
%         title('Fourier Basis:  imag(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,2);
%         imagesc(angle(F)); 
%         caxis([-pi pi]);
%         colorbar;
%         title('Fourier Basis:  angle(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,4);
%         imagesc(real(F'*F)); colorbar;
%         title('Orthogonality:  F*F''');
%         xlabel('Frequency');
%         ylabel('Frequency');
%     
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 3.2: Questions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   1) For a time-series X = [x1 ... xN], the Discrete Fourier Transform 
    %    (DFT) of W = F'X'. Every orthonormal basis can be thought of as a
    %    rotation of some set of data. In this case, how many data point
    %    are there, and what is the dimensionality of the data with respect 
    %    to rotation?
    %
    %    Hint: Remember that F is a complex basis. Factor this into
    %    considerations of dimensionality.
    %
    % 2) For X = [x1 ... xN], N/2 is considered to be the Nyquist
    %    frequency, the largest frequency that can be represented in a
    %    time-series of length N. Yet there would appear to be N/2 higher
    %    frequencies. Why is this not the case?
    %
    %    Let S be a [N N] matrix, with:
    %
    %       N = 10;
    %       w = [0:N-1]./N;
    %       t = [0:N-1];
    %       i = sqrt(-1);
    %       S(k,n) = sin(2*pi*i*w(k)*t(n));
    %
    %    Use imagesc to plot S, and compare the sinusoids with w(k) < N/2
    %    to those with w(k) > N/2. This effect is called aliasing. Describe
    %    the intuition behind aliasing.
    %
    % 3) Choose natural numbers k1 and k2, and construct three time-series: 
    %       
    %        s1 = sin(2*pi*i*w(k1)*t) + sin(2*pi*i*w(k2)*t);
    %        s2 = cos(2*pi*i*w(k1)*t) + cos(2*pi*i*w(k2)*t);
    %        s3 = sin(2*pi*i*(w(k1)+.1)*t) + sin(2*pi*i*(w(k2)+.1)*t);
    %
    %    Take the Fourier Transform of each using the fft algorithm:
    %
    %       Sk = fft(sk);
    %
    %   For each, plot the amplitude of Sk = abs(Sk), and the phase of 
    %   Sk = angle(Sk);
    %
    %   Describe the differences in each transformed series with respect to 
    %   amplitude and phase.
    %
    % 4) For vectors X = [x1 ... xN] and Y = [y1 ... yn], each with zero 
    %    mean, the covariance between X and Y, cov(X,Y) = X*Y', is simply
    %    the dot product between the two. 
    %
    %    For a basis B, consider the columns to be zero mean 'shapes'. In 
    %    the Fourier basis, the shapes are helices. What can be said about 
    %    the different shapes of a basis with respect to covariance? 
    %
    %    By a similar analogy, projecting data onto a single dimension is 
    %    akin to comparing it to a shape. What is the relationship between 
    %    the value of the transformed data along a single dimension and the 
    %    shape which represents that dimension?
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% BEGIN SECTION 4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 4: The Convolution Operator %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     Convolution is the fundamental operation in the domain of linear
%     systems. As it turns out, all linear systems can be described as the
%     convolution of an input function with a fixed transfer function, also
%     known as an impulse response function, Green's function or a kernel. 
%
%     Convolution is defined in the following way (Latex notation):
%
%     f * g  =  \int_\infty^\infty{ F(tau)g(t - tau)dtau };
%
%     Read, 'the integral from positive to negative infinity of f of tau
%     times g of t minus tau, dtau. I will denote convolution with an 
%     asterisk and two spaces, so as not to have it confused with 
%     multiplication.
%
%     Let F{f} be the Fourier transform of f(t), and inv(F{f}} = f be the 
%     inverse Fourier transform. Two important theorems can be stated. 
%     Together, they constitute a statement of the duality between
%     convolution and multiplication. 
%
%     1) f * g = inv(F{f}F{g})
%     
%     2) f g = inv(F{f} * F{g})
%
%     That is, convolution in the time domain is multiplication in the
%     frequency domain. Multiplication in the frequency domain is
%     convolution in the time domain. This is an extremely powerful pair of
%     theorems.
%
%     In the discrete domain, where f is a finite-length time-series, the
%     discrete Fourier transform (DFT) corresponds the to Fourier basis,
%     described above. In this case, convolution is circular. It wraps
%     around. Even so, the same equalities hold in both cases.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 4.1: Convolution %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %% Convolution is the continuous version of a sliding dot product of 
%     %% two functions, one flipped about the vertical axis. The following 
%     %% movie may help to clarify.
% 
%     %Intiializing convolution functions
%     clear; 
%     N = 100; a = .5;
%     t = linspace(0,10*pi,N);
%     dt = t(2)-t(1);
%     f = sin(t);
%     g = 10*a*t.*exp(-a*t);
%     fcg = conv(f,g);
%     
%     figure;
%     setP([334 136 939 564]);
%     subplot(2,3,1);
%         plot(t,f,'Color',[0 0 .8],'LineWidth',2);
%         A = axis; axis tight; A2 = axis; 
%         axis([A2(1:2) A(3:4)]);
%         setXT; setXTL;
%         setYT; setYTL;
%         xlabel('time')
%         title('Function f(t)');
%     subplot(2,3,4);
%         plot(t,g,'Color',[0 .4 .4],'LineWidth',2);
%         A = axis; axis tight; A2 = axis; 
%         axis([A2(1:2) A(3:4)]);
%         setXT; setXTL;
%         setYT; setYTL;
%         xlabel('time')
%         title('Function f(t)');
%     disp('Hold down RIGHT ARROW');
%         
%     k = 1;   
%     done = 0;
%     g = fliplr(g);
%     tau = t(end);     
%     A1 = [0 t(end) min([min(f) min(g)]) max([max(f) max(g)])];
%     A2 = [-t(end) 0 minmax(fcg)];
%  
%     for k = 1:length(t)
%        if(k == 1)
%        subplot(2,2,2);
%            plot(t,f,'Color',[0 0 .8],'LineWidth',2); hold on;
%            h1 = plot(t-tau,g,'Color',[0 .4 .4],'LineWidth',2); hold off;
%            axis(A1);
%            setXT; setXTL;
%            setYT; setYTL;
%            xlabel('time')
%            title('f(tau), g(t-tau)');
%        subplot(2,2,4);
%            h2 = plot(t(1:k)-t(end),fcg(1:k),'k','LineWidth',2);
%            axis(A2);
%            setXT; setXTL;
%            setYT; setYTL;
%            xlabel('time')
%            title('f * g');
%        else
%            set(h1,'XData',t-tau);
%            set(h2,'XData',t(1:k)-t(end));
%            set(h2,'YData',fcg(1:k));
%        end
%        pause;
%             
%        tau = tau - dt;
%        if(k == length(fcg))
%            done = 1;
%        end
%     end
%      
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 4.2: Duality %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     %% Convolution and multiplication are dual to each other with respect
%     %% to the time and frequency domains. Below shows an example using
%     %% circular convolution, the discrete analog of full convolution.
%
%     %Circular convolution and the Fourier Transform.
%     %Intiializing convolution functions
%     N = 100; a = .5;
%     t = linspace(0,10*pi,N);
%     dt = t(2)-t(1);
%     f = sin(t);
%     g = 10*a*t.*exp(-a*t);
% 
%     %Circular convolution in the time domain.
%     g = fliplr(g);
%     fcg1 = zeros(1,N);
%     ind = [N 1:N-1];
%     for k = 1:N
%         fcg1(k) = f*g(ind)';
%         ind = [ind(end) ind(1:end-1)];
%     end
% 
%     %Circular Concolution in the frequency domain.
%     N = 100; a = .5;
%     t = linspace(0,10*pi,N);
%     dt = t(2)-t(1);
%     f = sin(t);
%     g = 10*a*t.*exp(-a*t);
%     fcg2 = real(ifft(fft(f).*fft(g)));
% 
%     figure;
%     setP([339 210 560 418]);
%     plot(t,fcg1,'k','LineWidth',3); hold on;
%     plot(t,fcg2,'r','LineWidth',1); hold off;
%     legend('f * g (time domain)','f * g (frequency domain)');
%     axis tight; A = axis; 
%     axis([A(1:2) 1.5*(A(3:4)-mean(A(3:4)))+ mean(A(3:4))]);
%     setXT; setXTL;
%     setYT; setYTL;
%     xlabel('time')
%     title('f * g');
%     
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 4: Questions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   1) A proof of the convolution theorem is given on Mathworld:
    %
    %      http://mathworld.wolfram.com/ConvolutionTheorem.html
    %
    %      Write up the proof in Latex. Add in comments for each step,
    %      explaining what has been done. Try to explain the way in which
    %      each step serves to bring the current equation closer to the end
    %      goal.
    %
    %   2) Convolution has many desirable properties. Some of these are:
    %
    %      Associativity              :   f * (g * h) = (f * g) * h
    %
    %      Commutivity                :   f * g = g * f
    %
    %      Additive Distributivity    :   f * (g + h) = (f * g) + (f * h)
    %
    %      Multiplicative Identity    :   f * delta = f
    %
    %      Scalar Associativity       :   a(f * g) = (af) * g
    %
    %      Derivative Associativity   :   D[f * g] = D[f] * g
    %
    %      i)   Here 'delta' is the delta function, and 'D[]' is the 
    %           derivative operator. is the derivative operatorUsing the 
    %           above rules, Prove:
    %
    %           (af) * g = f * (ag)
    %
    %           D[f] * g = f * D[g]
    %
    %      ii)  Using the integral definition of convolution, prove the
    %           following properties about convolution:
    %
    %           Associativity
    %           Commutivity
    %           Additive Distributivity
    %           Multiplicative Identity
    %           Scalar Associativity
    %
    %           Most problems can be solved using basic facts about the
    %           linearity of the integration operator. Look up the dirac
    %           delta function, should you need any help on the
    %           multiplicative identity case.
    %
    %     iii)  Research Problem: Let R denote the real numbers: the 
    %           standard set of numbers which serve as the range and domain 
    %           of functions which limits, derivatives and integrals 
    %           operate on. Addition and multiplication over the real 
    %           numbers are an algebraic structure called a field. The 
    %           structure is denoted:
    %
    %           {R + x}
    %           
    %           A field has the following properties:
    %           
    %           + is associative
    %           + is commutative
    %           + has an inverse element
    %           + has an identity element
    %           x is associative
    %           x is commutative
    %           x has an inverse element
    %           x has an identity element
    %           x distributes over +
    %
    %           Let L denote a function space. For f in L, f:R -> R. 
    %           Consider {L + *} where * is convolution. 
    %
    %           a) Read the following page. What is the name of the
    %           algebraic structure {L + *}? 
    %
    %           Hint: What is it missing to be a field?
    %
    %           b) Not all functions are integrable. It stands to reason,
    %           that if f and g are in L, and the integral of f and g is
    %           not defined, f * g is not defined. Try to find the largest
    %           function space L such that, if f and g are in L, f * g 
    %           exists and is in L.
    %
    %           Hint: Look up 'Lp norm'. This should help with function
    %           spaces.
    %
    %     iv)*  Let RM denote the set of rotation matrices, and let * 
    %           denote matrix multiplication. What algebraic structure is 
    %           {RM *}?
    %
    %           Use wikipedia to search for algebraic structures. Matrix
    %           multiplication is not commutative in general. Is this true
    %           for rotation matrices?
    %      
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% BEGIN SECTION 4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 5: Linear Systems %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% START UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %% SECTION 5.1: Convolution %%
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     %% 
%     
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% %% END UNCOMMENT HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% SECTION 5: Questions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 
    %   1) 
    %      
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%